package com.dragonsblaze.client;

import java.util.HashMap;
import java.util.Map;

import com.dragonsblaze.client.util.Analytics;
import com.dragonsblaze.client.util.History;
import com.dragonsblaze.client.util.LogUtils;
import com.dragonsblaze.client.view.View;
import com.google.gwt.dom.client.Style.Overflow;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Controls the Application state. Allows maintaining a list of views, which are placed in the
 * assigned target panel when triggered.
 * 
 * @author aaditya
 */
public class Controller
{
	private Map<String, Widget> widgetMap = new HashMap<String, Widget>();

	private String viewToken = "", viewState = "";

	private static final String queryString = Window.Location.getQueryString();

	private Panel targetPanel;

	private static Controller INSTANCE;

	public static Controller initialize()
	{
		if (INSTANCE == null)
			INSTANCE = new Controller();
		return INSTANCE;
	}

	public static Controller getInstance()
	{
		return INSTANCE;
	}

	private Controller()
	{
		LogUtils.log("Browser History supported: " + History.supported);

		History.subscribe(new Command()
		{
			@Override
			public void execute()
			{
				fireCurrentState();
			}
		});

		String path = Window.Location.getPath();

		if (path.startsWith("/"))
			path = path.substring(1);

		String[] states = path.split("/");
		if (states.length >= 1)
			this.viewToken = states[0];
		if (states.length >= 2)
			this.viewState = states[1];
	}
	
	public String getViewToken()
	{
		return this.viewToken;
	}

	/* View */

	public void setTargetPanel(Panel targetPanel)
	{
		this.targetPanel = targetPanel;
	}

	/**
	 * Utility function to set CSS overflow value of target content panel, only if controller is
	 * instantiated.
	 */
	public static void setOverflow(Overflow overflow)
	{
		if (INSTANCE == null || INSTANCE.targetPanel == null)
			return;

		INSTANCE.targetPanel.getElement().getStyle().setOverflow(overflow);
	}

	public void addView(String token, View view)
	{
		widgetMap.put(token, view);
	}

	/* Application State */

	public void saveState(String viewToken, String viewState)
	{
		if (viewToken == null || viewState == null)
			LogUtils.error("Invalid argument: viewToken = " + viewToken + ", viewState = "
					+ viewState);

		LogUtils.log("saveState: " + viewToken + ", " + viewState);

		boolean updateURL = false;
		if (!this.viewToken.equals(viewToken))
		{
			updateURL = true;
			this.viewToken = viewToken;
		}
		if (!this.viewState.equals(viewState))
		{
			updateURL = true;
			this.viewState = viewState;
		}

		if (updateURL)
			updateURL(this.viewToken, this.viewState);
	}

	/**
	 * Restore app state from the given path, and optionally write the path to Window URL
	 */
	public boolean restoreState(String path, boolean save)
	{
		LogUtils.log("restoreState: " + path);
		if (path.startsWith("/"))
			path = path.substring(1);
		String[] states = path.split("/");
		String viewToken = "", viewState = "";
		if (states.length >= 1)
			viewToken = states[0];
		if (states.length >= 2)
			viewState = states[1];
		return restoreState(viewToken, viewState, save);
	}

	public boolean restoreState(String viewToken, String viewState, boolean save)
	{
		Widget widget = widgetMap.get(viewToken);

		if (widget == null)
		{
			LogUtils.error("View not found: " + viewToken);
			return false;
		}

		if (save)
			saveState(viewToken, viewState);

		targetPanel.clear();
		targetPanel.add(widget);

		if (!(widget instanceof View) || viewState == null)
		{
			return true;
		}
		View view = (View) widget;
		view.restoreState(viewState);
		return true;
	}

	/**
	 * Updates Window path
	 */
	private void updateURL(String viewToken, String viewState)
	{
		if (viewState == null)
			viewState = "";

		String path = "/" + viewToken;
		if (!viewState.isEmpty())
			path += "/" + viewState;

		if (History.supported)
			History.pushPath(path + queryString);

		LogUtils.log("updateURL: " + path);
		Analytics.trackPageview(path);
	}

	/**
	 * Restores current state from URL path without writing it back
	 */
	public void fireCurrentState()
	{
		LogUtils.log("fireCurrentState: " + Window.Location.getPath());
		restoreState(Window.Location.getPath(), false);
	}
}
